package merlin-lib

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Merlin_analysis.OcamldocSource

Sourceval associate_comments : source:string option -> after_only:bool -> (string * Ocaml_parsing.Location.t) list -> Ocaml_parsing.Location.t -> string option

associate_comments ~source ~after_only comments loc finds the doc comments in comments associated with the declaration at loc. source is the text of the document, which is used to help associate comments correctly. If source is None, some comments may be incorrectly associated.