package merlin-lib

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Env.UnscopedSource

  • alert dangerous It is unsafe to use the common [Path.same] function in contexts where [Unscoped.with_pairs] has been used to enrich the environment with unscoped equalities; [Path.equiv] must be used instead. See [ctype.ml] for an example of careful usage of [Unscoped], by locally shadowing [Path] to hide [Path.same].
Sourceval with_pairs : (Ident.Unscoped.t * Ident.Unscoped.t) list -> t -> t
Sourceval get_pairs : t -> (Ident.Unscoped.t * Ident.Unscoped.t) list
Sourceval path_equiv : t -> Path.t -> Path.t -> bool