package merlin-lib

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Ident.UnscopedSource

Unscoped defines a notion of identifier that are bound locally inside a module-dependant function type : (module M : S) -> t).

Those identifiers do not have scopes because they are bound locally and can be unified with each other to respect alpha-conversion.

Sourcetype t
Sourceval create : string -> t
Sourceval refresh : t -> t
Sourceval name : t -> string
Sourceval same : t -> t -> bool
Sourcetype change
Sourceval change_log : (change -> unit) ref
Sourceval undo_change : change -> unit
Sourcemodule Set : Set.S with type elt = t