package merlin-lib

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Out_type.Ident_conflictsSource

The Ident_conflicts module keeps track of conflicts arising when attributing names to identifiers and provides functions that can print explanations for these conflict in error messages

Sourceval exists : unit -> bool

exists() returns true if the current naming context renamed an identifier to avoid a name collision

Sourcetype explanation = {
  1. kind : Shape.Sig_component_kind.t;
  2. name : string;
  3. root_name : string;
  4. location : Ocaml_parsing.Location.t;
}
Sourceval list_explanations : unit -> explanation list

list_explanations() return the list of conflict explanations collected up to this point, and reset the list of collected explanations

Sourceval print_located_explanations : explanation list Merlin_utils.Format_doc.printer
Sourceval err_msg : unit -> Merlin_utils.Format_doc.doc option

err_msg () return an error message if there are pending conflict explanations at this point. It is often important to check for conflicts after all printing is done, thus the delayed nature of err_msg

Sourceval reset : unit -> unit