package merlin-lib

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Merlin_sherlodoc.Type_distanceSource

Calculate an approximation of the distance between two types.

Sourceval compute : query:Type_expr.t -> entry:Type_expr.t -> int

compute a b calculates an approximation of the distance between query and entry.